use strictures;

use FindBin qw/$Bin/;
use lib "$Bin/lib";

use Test::Most tests => 2;

use state51::Odoo;
use state51::Odoo::Test::Helpers 'build_odoo_client_with_env';

=head1 SYNOPSYS 00_login.t

=head1 DESCRIPTION

Additional login testing because t/lib/state51/Odoo/TestUnit.pm redefine some
Frontier::Client methods in a way that voids the tests: couldn't find a way to
fix them and test the state51::Odoo->login() feature in a proper way.

You need a running Odoo test instance that got both XML and JSON RPC APIs.

=cut

subtest 'We should be able to login with the XML-RPC interface' => sub {

    my $o = build_odoo_client_with_env();

    my $uid = $o->odoo_uid;
    ok $uid > 0, 'We managed to login (retrieve an uid)';
    cmp_ok $uid, '==', 86, '...and we retrieved the right login';

    # NOTE deliberate wrong password here
    my $password = 'MISTAKE';

    my $expected_to_fail_o = build_odoo_client_with_env(undef, $password);

    note 'The warning about login failure is expected';

    throws_ok {
        my $failed_uid = $expected_to_fail_o->odoo_uid;
    }
    'Moose::Exception::ValidationFailedForInlineTypeConstraint',
        '...and failing the Odoo API login with deliberate wrong password throw an exception';
};

subtest 'We should be able to login with the JSON-RPC interface' => sub {

    throws_ok {
        my $meh_rpc = build_odoo_client_with_env('meh'),    # Parameter is the interface_type (xml|json)
    }
    'Moose::Exception::ValidationFailedForTypeConstraint',
        '...and failing the state51::Odoo creation because we try to create a forbidden interface';

    my $json_rpc = build_odoo_client_with_env('json');

    ok $json_rpc;

    isa_ok $json_rpc->object_service, 'MojoX::JSONRPC2::HTTP';

    #my ($failed, $res, $error) = $json_rpc->object_service->call('read', { message => 'coucou' });

    my $uid = $json_rpc->odoo_uid;
    cmp_ok $uid, '==', 86, '...and we retrieved the right odoo user uid';
};

done_testing();

