package state51::Odoo::Test;

use strictures;

use FindBin qw/$Bin/;
use lib "$Bin/lib";

use Test::Most;

use state51::Odoo;
use state51::Odoo::Test::Helpers 'build_odoo_client_with_env';
use state51::Odoo::StockManagement::Constants qw/
    ODOO_WAREHOUSE_STOCK_MOVE_STOCK_QUANT_IDS_FIELD
    ODOO_WAREHOUSE_TEST_PRODUCT_ID
    /;

=head1 SYNOPSYS 02_json_rpc.t

=head1 DESCRIPTION

We want to make sure that the basics of Odoo / Platform stock management
synchronisation works with the JSON-RPC interface.

Those tests can be run with MOJO_CLIENT_DEBUG=1 to get the JSON-RPC client
traces.

=head1 TODO

=over 2

=item Subroutine-based unit tests

=item get_product_stock_quant() is very sensitive as we had complicated issues with it. Notably exception management

=back

=cut

my $o_json = build_odoo_client_with_env('json', undef);

# What we need: get the stock_quant id that correspond to a product and it's
# location, that's all
subtest 'Simulate the Platform / Odoo stock synchronisation' => sub {

    $o_json = build_odoo_client_with_env('json', undef);

    my $product_with_location = $o_json->get_product(ODOO_WAREHOUSE_TEST_PRODUCT_ID);
    isa_ok $product_with_location, 'HASH', 'get_product() returns a hash ref';
    ok $product_with_location->{location_id}, '...and the product got a location_id';
    ok $product_with_location->{id},          '...and the product got an id';

    my $location_id = $product_with_location->{location_id};
    # FIXME should be 685 (Greedbag if more products went out than went in)
    is $location_id, 76, '...and the location_id value is expected';

    my $stock_quant = $o_json->get_product_stock_quant(ODOO_WAREHOUSE_TEST_PRODUCT_ID, $location_id);

    isa_ok $stock_quant, 'HASH', 'get_product_stock_quant() returns a hash ref';
    ok $stock_quant->{create_date}, '...and the product stock quant got a create_date';
    ok $stock_quant->{id},          '...and the product stock quant got an id';

    my $common_params = {
        odoo_product_id     => ODOO_WAREHOUSE_TEST_PRODUCT_ID,
        odoo_stock_quant_id => $stock_quant->{id},
    };

    $common_params->{odoo_stock_quantity_adjustment} = 14;
    ok $o_json->do_reconciliation($common_params),
        'The stock reconciliation with a quantity of 10 is successful. Post extra-checks might be necessary. though';

    $common_params->{odoo_stock_quantity_adjustment} = -7;
    ok $o_json->do_reconciliation($common_params),
        'The stock reconciliation with a quantity of -5 is successful. Post extra-checks might be necessary. though';

    # TODO Get the current quantity and check it's correct
};

done_testing();

